@@ -1622,7 +1622,12 @@ class KohzuSeqCtl_Monochromator(Device):
 
 class ProcessController(Device):
     """
-    common parts of a process controller support
+    DEPRECATED (1.5.1): Use ``ophyd.PVPositioner`` instead.
+
+    For the controlled signal (such as temperature), use ``ophyd.PVPositioner``.
+    Include this within an ``ophyd.Device`` as appropriate.
+
+    Common parts of a process controller support.
 
     .. index:: Ophyd Device; ProcessController
 
@@ -1710,8 +1715,8 @@ class ProcessController(Device):
 
     @property
     def value(self):
-        """shortcut to self.signal.value"""
-        return self.signal.value
+        """shortcut to self.signal.get()"""
+        return self.signal.get()
 
     @property
     def settled(self):
@@ -1855,7 +1860,7 @@ def AD_setup_FrameType(prefix, scheme="NeXus"):
     the ``AD_FrameType_schemes`` dictionary, defining storage values for the
     fields of the EPICS ``mbbo`` record that you will be using.
 
-    see: https://github.com/BCDA-APS/use_bluesky/blob/master/lessons/sandbox/images_darks_flats.ipynb
+    see: https://nbviewer.jupyter.org/github/BCDA-APS/use_bluesky/blob/main/lessons/sandbox/images_darks_flats.ipynb
 
     EXAMPLE::
 
