@@ -58,13 +58,19 @@ inactive_deadline = 0  # sscan timeout, absolute time.time()
 
 
 class CommandFileReadError(IOError):
-    ...
+    """
+    Exception when reading a command file.
+
+    .. index:: Bluesky Exception; CommandFileReadError
+    """
 
 
 def addDeviceDataAsStream(devices, label):
     """
     plan: add an ophyd Device as an additional document stream
 
+    .. index:: Bluesky Plan; addDeviceDataAsStream
+
     Use this within a custom plan, such as this example::
 
         from apstools.plans import addDeviceStream
@@ -92,6 +98,8 @@ def execute_command_list(filename, commands, md=None):
     """
     plan: execute the command list
 
+    .. index:: Bluesky Plan; execute_command_list
+
     The command list is a tuple described below.
 
     * Only recognized commands will be executed.
@@ -225,6 +233,8 @@ def lineup(
     """
     lineup and center a given axis, relative to current position
 
+    .. index:: Bluesky Plan; lineup
+
     PARAMETERS
 
     counter
@@ -372,6 +382,8 @@ def nscan(detectors, *motor_sets, num=11, per_step=None, md=None):
     """
     Scan over ``n`` variables moved together, each in equally spaced steps.
 
+    .. index:: Bluesky Plan; nscan
+
     PARAMETERS
 
     detectors
@@ -660,6 +672,8 @@ def run_command_file(filename, md=None):
     """
     plan: execute a list of commands from a text or Excel file
 
+    .. index:: Bluesky Plan; run_command_file
+
     * Parse the file into a command list
     * yield the command list to the RunEngine (or other)
 
@@ -686,6 +700,8 @@ def snapshot(obj_list, stream="primary", md=None):
     """
     bluesky plan: record current values of list of ophyd signals
 
+    .. index:: Bluesky Plan; snapshot
+
     PARAMETERS
 
     obj_list
@@ -814,6 +830,8 @@ def sscan_1D(
     """
     simple 1-D scan using EPICS synApps sscan record
 
+    .. index:: Bluesky Plan; sscan_1D
+
     assumes the sscan record has already been setup properly for a scan
 
     PARAMETERS
@@ -965,7 +983,11 @@ def sscan_1D(
 
 
 class TuneResults(Device):
-    """because bps.read() needs a Device or a Signal)"""
+    """
+    Provides bps.read() as a Device
+
+    .. index:: Bluesky Device; TuneResults
+    """
 
     tune_ok = Component(Signal)
     initial_position = Component(Signal)
@@ -1009,6 +1031,8 @@ class TuneAxis(object):
     """
     tune an axis with a signal
 
+    .. index:: Bluesky Device; TuneAxis
+
     This class provides a tuning object so that a Device or other entity
     may gain its own tuning process, keeping track of the particulars
     needed to tune this device again.  For example, one could add
@@ -1071,6 +1095,8 @@ class TuneAxis(object):
         """
         Bluesky plan to execute one pass through the current scan range
 
+        .. index:: Bluesky Plan; TuneAxis.tune
+
         Scan self.axis centered about current position from
         ``-width/2`` to ``+width/2`` with ``num`` observations.
         If a peak was detected (default check is that max >= 4*min),
@@ -1320,6 +1346,8 @@ def tune_axes(axes):
     """
     Bluesky plan to tune a list of axes in sequence
 
+    .. index:: Bluesky Plan; tune_axes
+
     EXAMPLE
 
     Sequentially, tune a list of preconfigured axes::
