@@ -4,11 +4,12 @@ Ophyd support for synApps configuration of EPICS records
 Support the default structures as provided by the
 synApps template XXX IOC.
 
-EXAMPLES:;
+EXAMPLES::
 
     import apstools.synApps
-    scans = apstools.synApps.SscanDevice("xxx:", name="scans")
     calcs = apstools.synApps.userCalcsDevice("xxx:", name="calcs")
+    scans = apstools.synApps.SscanDevice("xxx:", name="scans")
+    xxxstats = apstools.synApps.IocStatsDevice("xxx:", name="xxxstats")
 
     calc1 = calcs.calc1
     apstools.synApps.swait_setup_random_number(calc1)
@@ -21,21 +22,22 @@ Compare this effort with a similar project:
 https://github.com/klauer/recordwhat
 """
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # :author:    Pete R. Jemian
 # :email:     jemian@anl.gov
-# :copyright: (c) 2017-2020, UChicago Argonne, LLC
+# :copyright: (c) 2017-2021, UChicago Argonne, LLC
 #
 # Distributed under the terms of the Creative Commons Attribution 4.0 International Public License.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 
 from .asyn import *
 from .busy import *
 from .calcout import *
 from .epid import *
+from .iocstats import *
 from .save_data import *
 from .sscan import *
 from .swait import *
