@@ -131,7 +131,7 @@ class CalcoutRecord(EpicsRecordFloatFields, EpicsRecordDeviceCommonAll):
 
     @property
     def value(self):
-        return self.calculated_value.value
+        return self.calculated_value.get()
 
     def reset(self):
         """set all fields to default values"""
